# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

from typing import Dict, List, Optional

import urllib
import yaml
from tuxrun import templates
from tuxrun.devices import Device
from tuxrun.exceptions import InvalidArgument
from tuxrun.utils import compression, notnone, slugify


class FVPDevice(Device):
    flag_use_pre_run_cmd = True

    def device_dict(self, context):
        return templates.devices().get_template("fvp.yaml.jinja2").render(**context)


class AEMvAFVPDevice(FVPDevice):
    name = "fvp-aemva"

    enable_network: bool = True
    flag_cache_rootfs = True

    bl1 = "https://storage.tuxboot.com/buildroot/fvp-aemva/bl1.bin"
    dtb = "https://storage.tuxboot.com/buildroot/fvp-aemva/fvp-base-revc.dtb"
    fip = "https://storage.tuxboot.com/buildroot/fvp-aemva/fip.bin"
    kernel = "https://storage.tuxboot.com/buildroot/fvp-aemva/Image"
    rootfs = "https://storage.tuxboot.com/buildroot/fvp-aemva/rootfs.ext4.zst"
    uefi = "https://storage.tuxboot.com/buildroot/fvp-aemva/edk2-flash.img"

    def validate(
        self,
        bl1,
        boot_args,
        commands,
        dtb,
        fip,
        kernel,
        rootfs,
        uefi,
        overlays,
        parameters,
        prompt,
        modules,
        enable_network,
        tests,
        **kwargs,
    ):
        invalid_args = ["--" + k.replace("_", "-") for k in kwargs if kwargs[k]]
        if len(invalid_args) > 0:
            raise InvalidArgument(
                f"Invalid option(s) for fvp devices: {', '.join(sorted(invalid_args))}"
            )

        if boot_args and '"' in boot_args:
            raise InvalidArgument('argument --boot-args should not contains "')
        if prompt and '"' in prompt:
            raise InvalidArgument('argument --prompt should not contains "')
        if modules and compression(modules[0]) not in [("tar", "gz"), ("tar", "xz")]:
            raise InvalidArgument(
                "argument --modules should be a .tar.gz, tar.xz or .tgz"
            )

        for test in tests:
            test.validate(device=self, parameters=parameters, **kwargs)

    def default(self, options) -> None:
        options.bl1 = notnone(options.bl1, self.bl1)
        options.dtb = notnone(options.dtb, self.dtb)
        options.fip = notnone(options.fip, self.fip)
        options.kernel = notnone(options.kernel, self.kernel)
        options.rootfs = notnone(options.rootfs, self.rootfs)
        options.uefi = notnone(options.uefi, self.uefi)

    def definition(self, **kwargs):
        kwargs = kwargs.copy()

        kwargs["no_network"] = not kwargs["enable_network"]

        # Options that can be updated
        if kwargs["tux_prompt"]:
            kwargs["tux_prompt"] = [kwargs["tux_prompt"]]
        else:
            kwargs["tux_prompt"] = []

        kwargs["command_name"] = slugify(
            kwargs.get("parameters").get("command-name", "command")
        )

        # render the template
        tests = [
            t.render(
                arch="arm64",
                device=kwargs["device"],
                commands=kwargs["commands"],
                command_name=kwargs["command_name"],
                tmpdir=kwargs["tmpdir"],
                overlays=kwargs["overlays"],
                parameters=kwargs["parameters"],
                test_definitions=kwargs["test_definitions"],
            )
            for t in kwargs["tests"]
        ]
        return templates.jobs().get_template("fvp-aemva.yaml.jinja2").render(
            **kwargs
        ) + "".join(tests)

    def extra_assets(self, tmpdir, dtb, kernel, tux_boot_args, **kwargs):
        dtb = notnone(dtb, self.dtb).split("/")[-1]
        kernel = notnone(kernel, self.kernel).split("/")[-1]
        # Drop the extension if the kernel is compressed. LAVA will decompress it for us.
        if compression(kernel)[1]:
            kernel = kernel[: -1 - len(compression(kernel)[1])]
        (tmpdir / "startup.nsh").write_text(
            f"{kernel} dtb={dtb} systemd.log_level=warning {tux_boot_args + ' ' if tux_boot_args else ''}console=ttyAMA0 earlycon=pl011,0x1c090000 root=/dev/vda ip=dhcp",
            encoding="utf-8",
        )
        return [f"file://{tmpdir / 'startup.nsh'}"]


class MorelloFVPDevice(FVPDevice):
    mandatory = [
        "ap_romfw",
        "mcp_fw",
        "mcp_romfw",
        "rootfs",
        "scp_fw",
        "scp_romfw",
        "fip",
    ]

    prompts: List[str] = []
    auto_login: Dict[str, str] = {}
    boot_timeout = 20
    kernel_start_message: Optional[str] = None
    support_tests = False
    rootfs: Optional[str] = None

    def validate(
        self,
        ap_romfw,
        mcp_fw,
        mcp_romfw,
        rootfs,
        scp_fw,
        scp_romfw,
        overlays,
        parameters,
        tests,
        fip,
        **kwargs,
    ):
        invalid_args = ["--" + k.replace("_", "-") for k in kwargs if kwargs[k]]
        if len(invalid_args) > 0:
            raise InvalidArgument(
                f"Invalid option(s) for fvp-morello devices: {', '.join(sorted(invalid_args))}"
            )

        args = locals()
        missing_args = [
            "--" + k.replace("_", "-") for k in self.mandatory if not args[k]
        ]
        if len(missing_args) > 0:
            raise InvalidArgument(
                f"Missing option(s) for fvp devices: {', '.join(sorted(missing_args))}"
            )

        if tests and not self.support_tests:
            raise InvalidArgument("Tests are not supported on this device")

        if self.rootfs and rootfs:
            raise InvalidArgument("Invalid option for this fvp device: --rootfs")

        for test in tests:
            test.validate(device=self, parameters=parameters, **kwargs)

    def default(self, options) -> None:
        if self.rootfs:
            options.rootfs = notnone(options.rootfs, self.rootfs)

    def definition(self, **kwargs):
        kwargs = kwargs.copy()

        # Options that can *not* be updated
        kwargs["prompts"] = self.prompts.copy()
        kwargs["auto_login"] = self.auto_login.copy()
        kwargs["kernel_start_message"] = self.kernel_start_message
        kwargs["support_tests"] = self.support_tests
        kwargs["boot_timeout"] = self.boot_timeout

        # render the template
        tests = [
            t.render(
                tmpdir=kwargs["tmpdir"],
                parameters=kwargs["parameters"],
                prompts=kwargs["prompts"],
            )
            for t in kwargs["tests"]
        ]
        return (
            templates.jobs().get_template("fvp-morello.yaml.jinja2").render(**kwargs)
            + "\n"
            + "".join(tests)
        )


class FVPMorelloAndroid(MorelloFVPDevice):
    name = "fvp-morello-android"

    prompts = ["console:/ "]
    support_tests = True


class FVPMorelloBusybox(MorelloFVPDevice):
    name = "fvp-morello-busybox"

    prompts = ["/ # "]
    support_tests = True
    virtiop9_path = "/etc"


class FVPMorelloDebian(MorelloFVPDevice):
    name = "fvp-morello-debian"

    prompts = ["morello-deb:~#", "root@morello:~#"]
    support_tests = True
    auto_login = {
        "login_prompt": "login:",
        "username": "root",
        "password_prompt": "Password:",
        "password": "morello",
    }


class FVPMorelloBaremetal(MorelloFVPDevice):
    name = "fvp-morello-baremetal"

    mandatory = ["ap_romfw", "mcp_fw", "mcp_romfw", "scp_fw", "scp_romfw", "fip"]
    prompts = ["hello"]
    kernel_start_message = "Booting Trusted Firmware"


class FVPMorelloOE(MorelloFVPDevice):
    name = "fvp-morello-oe"

    prompts = ["root@morello-fvp:~# "]
    support_tests = True


class FVPMorelloUbuntu(MorelloFVPDevice):
    name = "fvp-morello-ubuntu"

    mandatory = ["ap_romfw", "mcp_fw", "mcp_romfw", "scp_fw", "scp_romfw", "fip"]

    prompts = ["morello@morello-server:"]
    auto_login = {
        "login_prompt": "morello-server login:",
        "username": "morello",
        "password_prompt": "Password:",
        "password": "morello",
    }
    boot_timeout = 60
    rootfs = "https://storage.tuxboot.com/fvp-morello-ubuntu/ubuntu.satadisk.xz"


class FVPMorelloGrub(MorelloFVPDevice):
    name = "fvp-morello-grub"

    kernel_start_message = "Press enter to boot the selected OS"
    prompts = ["highlighted entry will be executed"]
    support_tests = True


class FVPLAVA(FVPDevice):
    name = "fvp-lava"

    def validate(self, job_definition, **kwargs):
        parsed_url = urllib.parse.urlparse(job_definition)
        job_definition = urllib.parse.unquote(parsed_url.path)
        with open(job_definition, "r") as job_file:
            try:
                # Load yaml and dump data as string to verify that
                # lava job definition is valid
                yaml_data = yaml.dump(yaml.safe_load(job_file))
                self.job_definition = yaml_data
            except Exception:
                raise InvalidArgument("Unable to load LAVA job definition")
        return

    def default(self, options) -> None:
        ...  # noqa: E704

    def definition(self, **kwargs):
        return self.job_definition



#!/usr/bin/python3
# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

import argparse
import sys
from pathlib import Path

from tuxrun import __version__
from tuxrun.assets import get_rootfs, get_test_definitions
from tuxrun.devices import Device
from tuxrun.tests import Test
from tuxrun.tuxmake import TuxBuildBuild, TuxMakeBuild
from tuxrun.utils import ProgressIndicator, pathurlnone


###########
# Helpers #
###########
def filter_options(options):
    keys = [
        "device",
        "tuxbuild",
        "tuxmake",
        "timeouts",
        "runtime",
        "image",
        "qemu_image",
        "qemu_binary",
        "cache_dir",
        "save_outputs",
        "log_file",
        "log_file_html",
        "log_file_text",
        "log_file_yaml",
        "metadata",
        "results",
        "results_hooks",
        "shell",
        "debug",
        "lava_definition",
        "shared",
    ]
    return {k: getattr(options, k) for k in vars(options) if k not in keys}


def filter_artefacts(options):
    keys = [
        "ap-romfw",
        "bios",
        "bl1",
        "dtb",
        "fip",
        "kernel",
        "mcp-fw",
        "mcp-romfw",
        "modules",
        "overlays",
        "rootfs",
        "scp-fw",
        "scp-romfw",
        "uefi",
    ]
    return {k: getattr(options, k) for k in vars(options) if k in keys}


def tuxbuild_url(s):
    try:
        return TuxBuildBuild(s.rstrip("/"))
    except TuxBuildBuild.Invalid as e:
        raise argparse.ArgumentTypeError(str(e))


def tuxmake_directory(s):
    try:
        return TuxMakeBuild(s)
    except TuxMakeBuild.Invalid as e:
        raise argparse.ArgumentTypeError(str(e))


###########
# Actions #
###########
class ListDevicesAction(argparse.Action):
    def __init__(
        self, option_strings, help, dest=argparse.SUPPRESS, default=argparse.SUPPRESS
    ):
        super().__init__(option_strings, dest=dest, default=default, nargs=0, help=help)

    def __call__(self, parser, namespace, values, option_string=None):
        parser._print_message(
            "\n".join([d.name for d in Device.list()]) + "\n", sys.stdout
        )
        parser.exit()


class ListTestsAction(argparse.Action):
    def __init__(
        self, option_strings, help, dest=argparse.SUPPRESS, default=argparse.SUPPRESS
    ):
        super().__init__(option_strings, dest=dest, default=default, nargs=0, help=help)

    def __call__(self, parser, namespace, values, option_string=None):
        parser._print_message("\n".join(Test.list()) + "\n", sys.stdout)
        parser.exit()


class KeyValueAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        for value in values:
            key, *value = value.split("=")
            getattr(namespace, self.dest)[key] = "=".join(value)


class KeyValueIntAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        KEYS = ["deploy", "boot"] + Test.list()
        for value in values:
            try:
                key, value = value.split("=")
            except ValueError:
                raise argparse.ArgumentError(
                    self, f"Invalid format for '{value}' timeout"
                )
            if key not in KEYS:
                raise argparse.ArgumentError(self, f"Invalid timeout '{key}'")
            try:
                getattr(namespace, self.dest)[key] = int(value)
            except ValueError:
                raise argparse.ArgumentError(
                    self, f"Invalid value for {key} timeout: '{value}'"
                )


class OneTwoPathAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        if len(values) == 1:
            values = [values[0], "/"]
        elif len(values) > 2:
            raise argparse.ArgumentError(
                self,
                "takes one or two arguments, first should be a URL and second the destination.",
            )
        try:
            values[0] = pathurlnone(values[0])
        except argparse.ArgumentTypeError as exc:
            raise argparse.ArgumentError(self, str(exc))
        if isinstance(getattr(namespace, self.dest), list):
            getattr(namespace, self.dest).append(values)
        else:
            setattr(namespace, self.dest, values)


class SharedAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        if values is None:
            return
        if len(values) == 1:
            values = [values[0], "/mnt/tuxrun"]
        if len(values) > 2:
            raise argparse.ArgumentError(
                self,
                "takes zero, one or two arguments, first is the source and the second the destination. The later is optional.",
            )
        setattr(namespace, self.dest, values)


class UpdateCacheAction(argparse.Action):
    def __init__(
        self, option_strings, help, dest=argparse.SUPPRESS, default=argparse.SUPPRESS
    ):
        super().__init__(option_strings, dest=dest, default=default, nargs=0, help=help)

    def __call__(self, parser, namespace, values, option_string=None):
        print("Updating local cache:")
        print("* Rootfs:")
        for device in [d for d in Device.list() if d.flag_cache_rootfs]:
            print(f"  * {device.name}")
            get_rootfs(
                device, progress=ProgressIndicator.get("Downloading root filesystem")
            )
        print("* Test definitions")
        get_test_definitions(
            progress=ProgressIndicator.get("Downloading test definitions")
        )
        parser.exit()


##########
# Setups #
##########
def setup_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(prog="tuxrun", description="TuxRun")

    parser.add_argument(
        "--version", action="version", version=f"%(prog)s, {__version__}"
    )

    group = parser.add_argument_group("listing")
    group.add_argument(
        "--list-devices", action=ListDevicesAction, help="List available devices"
    )
    group.add_argument(
        "--list-tests", action=ListTestsAction, help="List available tests"
    )

    group = parser.add_argument_group("cache")
    group.add_argument(
        "--update-cache", action=UpdateCacheAction, help="Update assets cache"
    )

    group = parser.add_argument_group("artefacts")

    def artefact(name):
        group.add_argument(
            f"--{name}",
            default=None,
            metavar="URL",
            type=pathurlnone,
            help=f"{name} URL",
        )

    artefact("ap-romfw")
    artefact("bios")
    artefact("bl1")
    artefact("ssh-identity-file")
    artefact("dtb")
    artefact("fip")
    artefact("job-definition")
    artefact("kernel")
    artefact("mcp-fw")
    artefact("mcp-romfw")
    group.add_argument(
        "--modules",
        default=None,
        type=str,
        help="modules URL and optionally PATH to extract the modules, default PATH '/'",
        action=OneTwoPathAction,
        nargs="+",
    )
    group.add_argument(
        "--overlay",
        default=[],
        type=str,
        help="Tarball with overlay and optionally PATH to extract the tarball, default PATH '/'. Overlay can be specified multiple times",
        action=OneTwoPathAction,
        nargs="+",
        dest="overlays",
    )
    group.add_argument(
        "--partition",
        default=None,
        metavar="NUMBER",
        type=int,
        help="rootfs partition number",
    )
    artefact("rootfs")
    artefact("scp-fw")
    artefact("scp-romfw")
    group.add_argument(
        "--ssh-host",
        default=None,
        metavar="HOST ADDR",
        type=str,
        help="ssh host address, applicable to ssh-device",
    )
    group.add_argument(
        "--ssh-port",
        default=None,
        metavar="NUMBER",
        type=int,
        help="ssh port number. Defaults to 22, applicable to ssh-device",
    )
    group.add_argument(
        "--ssh-prompt",
        default=None,
        metavar="STRING",
        type=str,
        help="ssh prompt to expect, applicable to ssh-device",
    )
    group.add_argument(
        "--ssh-user",
        default=None,
        metavar="USERNAME",
        type=str,
        help="ssh username, applicable to ssh-device",
    )
    group.add_argument(
        "--tuxbuild",
        metavar="URL",
        default=None,
        type=tuxbuild_url,
        help="URL of a TuxBuild build",
    )
    group.add_argument(
        "--tuxmake",
        metavar="DIRECTORY",
        default=None,
        type=tuxmake_directory,
        help="directory containing a TuxMake build",
    )
    artefact("uefi")
    group.add_argument(
        "--fvp-ubl-license",
        default=None,
        metavar="FVP UBL License",
        type=str,
        help="UBL License to be passed to FVP that need User Based License. Applicable to FVP device type only",
    )

    group = parser.add_argument_group("secrets")
    group.add_argument(
        "--secrets",
        metavar="K=V",
        default={},
        type=str,
        help="job secrets as key=value",
        action=KeyValueAction,
        nargs="+",
    )

    group = parser.add_argument_group("test parameters")
    group.add_argument(
        "--parameters",
        metavar="K=V",
        default={},
        type=str,
        help="test parameters as key=value",
        action=KeyValueAction,
        nargs="+",
    )
    group.add_argument(
        "--tests",
        nargs="+",
        default=[],
        metavar="T",
        help="test suites",
        choices=Test.list(),
        action="extend",
    )
    group.add_argument(
        "--shell",
        action="store_true",
        help="Start a shell in the VM",
    )
    group.add_argument(
        "commands",
        nargs="*",
        help="Space separated list of commands to run inside the VM",
    )

    group = parser.add_argument_group("run options")
    group.add_argument(
        "--device",
        default=None,
        metavar="NAME",
        help="Device type",
        choices=[d.name for d in Device.list()],
    )
    group.add_argument(
        "--boot-args", default=None, metavar="ARGS", help="extend boot arguments"
    )
    group.add_argument(
        "--prompt", default=None, metavar="PROMPT", help="extra console prompt"
    )
    group.add_argument(
        "--timeouts",
        metavar="K=V",
        default={},
        type=str,
        help="timeouts in minutes as action=duration",
        action=KeyValueIntAction,
        nargs="+",
    )
    group.add_argument(
        "--enable-kvm",
        default=False,
        action="store_true",
        help="Enable kvm, only possible if host and QEMU system are the same",
    )
    group.add_argument(
        "--enable-trustzone",
        default=False,
        action="store_true",
        help="Enable trustzone, applicable to QEMU arm64 device only",
    )

    group.add_argument(
        "--enable-network",
        default=False,
        action="store_true",
        help="Enable network",
    )

    group = parser.add_argument_group("runtime")
    group.add_argument(
        "--runtime",
        default="podman",
        metavar="RUNTIME",
        choices=["docker", "null", "podman"],
        help="Runtime",
    )
    group.add_argument(
        "--image",
        default="docker.io/linaro/tuxrun-dispatcher:latest",
        help="Image to use",
    )
    group.add_argument(
        "--qemu-image", default=None, help="Use qemu from the given container image"
    )

    group.add_argument(
        "--qemu-binary",
        default=None,
        type=Path,
        help="Use qemu from the given path",
    )

    group = parser.add_argument_group("output")
    group.add_argument(
        "--cache-dir",
        default=None,
        type=Path,
        help="Change the cache directory for storing log files, default: XDG_CACHE_DIR",
    )
    group.add_argument(
        "--save-outputs",
        default=False,
        action="store_true",
        help="Automatically save every outputs",
    )
    group.add_argument("--log-file", default=None, type=Path, help="Store logs to file")
    group.add_argument(
        "--log-file-html", default=None, type=Path, help="Store logs to file as HTML"
    )
    group.add_argument(
        "--log-file-text", default=None, type=Path, help="Store logs to file as text"
    )
    group.add_argument(
        "--log-file-yaml", default=None, type=Path, help="Store logs to file as YAML"
    )
    group.add_argument(
        "--metadata", default=None, type=Path, help="Save test metadata to file (JSON)"
    )
    group.add_argument(
        "--results-hook",
        type=str,
        action="append",
        dest="results_hooks",
        metavar="COMMAND",
        help="Execute COMMAND after the run is finished, if the run is successful. Can be specified multiple times. The command is executed with the run output directory (i.e. where all the artifacts are) as working directory. If any results hook fails, tuxrun exits with a non-zero exit code.",
    )
    group.add_argument(
        "--results", default=None, type=Path, help="Save test results to file (JSON)"
    )
    group.add_argument(
        "--lava-definition",
        default=False,
        action="store_true",
        help="Save the LAVA definition.yaml file",
    )
    group.add_argument(
        "--shared",
        default=None,
        type=str,
        help="Directory to share with the device",
        action=SharedAction,
        nargs="*",
    )

    group = parser.add_argument_group("debugging")
    group.add_argument(
        "--debug",
        default=False,
        action="store_true",
        help="Print more debug information about tuxrun",
    )

    return parser



# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

from tuxrun import templates
from tuxrun.devices import Device
from tuxrun.exceptions import InvalidArgument
from tuxrun.utils import notnone, slugify


class SSHDevice(Device):
    name = "ssh-device"
    ssh_port = 22

    def validate(
        self,
        commands,
        tests,
        parameters,
        overlays,
        ssh_host,
        ssh_port,
        ssh_prompt,
        ssh_user,
        ssh_identity_file,
        **kwargs,
    ):
        invalid_args = ["--" + k.replace("_", "-") for k in kwargs if kwargs[k]]
        if len(invalid_args) > 0:
            raise InvalidArgument(
                f"Invalid option(s) for ssh device: {', '.join(sorted(invalid_args))}"
            )
        self.ssh_host = ssh_host
        self.ssh_user = ssh_user
        self.ssh_identity_file = ssh_identity_file.replace("file://", "")

    def default(self, options) -> None:
        options.ssh_port = notnone(options.ssh_port, self.ssh_port)

    def definition(self, **kwargs):
        # Options that can be updated
        if kwargs["ssh_prompt"]:
            kwargs["ssh_prompt"] = [kwargs["ssh_prompt"]]
        else:
            kwargs["ssh_prompt"] = []

        kwargs["command_name"] = slugify(
            kwargs.get("parameters").get("command-name", "command")
        )

        # render the template
        tests = [
            t.render(
                arch="arm64",
                commands=kwargs["commands"],
                command_name=kwargs["command_name"],
                device=kwargs["device"],
                tmpdir=kwargs["tmpdir"],
                ssh_prompt=kwargs["ssh_prompt"],
                overlays=kwargs["overlays"],
                parameters=kwargs["parameters"],
                test_definitions=kwargs["test_definitions"],
            )
            for t in kwargs["tests"]
        ]
        return templates.jobs().get_template("ssh.yaml.jinja2").render(
            **kwargs
        ) + "".join(tests)

    def device_dict(self, context):
        context["ssh_host"] = self.ssh_host
        context["ssh_user"] = self.ssh_user
        context["ssh_port"] = self.ssh_port
        context["ssh_identity_file"] = f"{self.ssh_identity_file}"
        context["lava_test_results_dir"] = "/tmp/lava-%s"
        return templates.devices().get_template("ssh.yaml.jinja2").render(**context)



import json
import os
from pathlib import Path

import pytest

from tuxrun.results import Results
from tuxrun.tests import Test

BASE = (Path(__file__) / "..").resolve()


def test_returns_0_by_default():
    results = Results([], {})
    results.__data__ = {"lava": {}}
    assert results.ret() == 0


def gen_test(name, result, suite_name="mytestsuite"):
    return f'{{ "lvl": "results", "msg": {{"definition": "{suite_name}", "case": "{name}", "result": "{result}"}}}}'


def test_returns_0_with_no_failures():
    t1 = Test(timeout=None)
    t1.name = "mytestsuite"
    results = Results([t1], {})
    results.parse(gen_test("test1", "pass"))
    results.parse(gen_test("test2", "pass"))
    results.parse(gen_test("job", "pass", suite_name="lava"))
    assert results.ret() == 0


def test_returns_1_on_failure():
    results = Results([], {})
    results.parse(gen_test("test1", "pass", suite_name="lava"))
    results.parse(gen_test("test2", "fail", suite_name="lava"))
    assert results.ret() == 1


def test_returns_2_on_missing_test():
    t1 = Test(timeout=None)
    t1.name = "mytestsuite"
    results = Results([t1], {})
    results.parse(gen_test("test1", "pass", suite_name="lava"))
    assert results.ret() == 2


def test_returns_invalid_logs():
    results = Results([], {})
    results.parse("{")
    results.parse('{ "lvl": "results", "msg": {"case": "tux", "result": "pass"}}')


def test_data():
    results = Results([], {})
    results.parse(gen_test("test1", "pass"))
    assert results.data["mytestsuite"]["test1"]["result"] == "pass"


@pytest.mark.parametrize(
    "name,testsuite",
    [
        ("canceled-1", "ltp-smoke"),
        ("fail-empty", "ltp-smoke"),
        ("fail-1", "ltp-containers"),
        ("fail-2", "ltp-mm"),
        ("fail-3", "ltp-syscalls"),
        ("fail-4", "libhugetlbfs"),
        ("fail-5", "ltp-tracing"),
        ("fail-6", "ltp-smoke"),
        ("pass-1", "ltp-io"),
        ("pass-2", "ltp-smoke"),
    ],
)
def test_results_parsing(name, testsuite):
    logs = (BASE / "logs" / (name + ".yaml")).read_text(encoding="utf-8").strip("\n")

    results = Results(
        [Test.select(testsuite)],
        {
            "kernel": "https://example.com/bzImage",
            "modules": "https://example.com/modules.tar.xz",
            "rootfs": "https://example.com/rootfs.ext4.xz",
            "overlays": [["https://example.com/ltp.tar.xz", "/"]],
        },
    )
    assert results.__artefacts__ == {
        "kernel": "https://example.com/bzImage",
        "rootfs": "https://example.com/rootfs.ext4.xz",
        "modules": "https://example.com/modules.tar.xz",
        "overlays": [["https://example.com/ltp.tar.xz", "/"]],
        "overlay-00": "https://example.com/ltp.tar.xz",
    }
    for line in logs.split("\n"):
        results.parse(line[2:])

    # Full results
    if os.environ.get("TUXRUN_RENDER"):
        (BASE / "results" / (name + ".json")).write_text(
            json.dumps(results.data), encoding="utf-8"
        )
    data = (BASE / "results" / (name + ".json")).read_text(encoding="utf-8")
    assert results.data == json.loads(data)

    if os.environ.get("TUXRUN_RENDER"):
        (BASE / "results" / (name + "-metadata.json")).write_text(
            json.dumps(results.metadata), encoding="utf-8"
        )
    data = (BASE / "results" / (name + "-metadata.json")).read_text(encoding="utf-8")
    assert results.metadata == json.loads(data)



# vim: set ts=4
#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT

from tuxrun.tests import Test


class RCUTorture(Test):
    devices = ["qemu-*", "fvp-aemva", "avh-imx93", "avh-rpi4b"]
    name = "rcutorture"
    timeout = 15
    need_test_definition = True

    def render(self, **kwargs):
        kwargs["name"] = self.name
        kwargs["timeout"] = self.timeout

        return self._render("rcutorture.yaml.jinja2", **kwargs)



# vim: set ts=4
#
# Copyright 2024-present Linaro Limited
#
# SPDX-License-Identifier: MIT

from tuxrun.tests import Test


class TFATests(Test):
    devices = ["qemu-arm64"]
    name = "tfa-tests"
    timeout = 30

    def render(self, **kwargs):
        kwargs["name"] = self.name
        kwargs["timeout"] = self.timeout

        return self._render("tfa-tests.yaml.jinja2", **kwargs)



# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

import requests
from requests.adapters import HTTPAdapter
from requests.packages import urllib3  # type: ignore
from requests.packages.urllib3.util.retry import Retry  # type: ignore

timeout = 60


def get_session(*, retries):
    session = requests.Session()
    if urllib3.__version__ >= "1.26":
        allowed_methods = "allowed_methods"
    else:
        allowed_methods = "method_whitelist"  # pragma: no cover
    retry_strategy = Retry(
        total=retries,
        status_forcelist=[413, 429, 500, 503, 504],
        backoff_factor=1,
        **{allowed_methods: ["HEAD", "OPTIONS", "GET", "POST"]},
    )
    adapter = HTTPAdapter(max_retries=retry_strategy)
    session.mount("https://", adapter)
    session.mount("http://", adapter)
    return session


def requests_get(*args, **kwargs):
    session = get_session(retries=8)
    return session.get(*args, timeout=timeout, **kwargs)



# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

import os
from pathlib import Path


def _resolve(var, default):
    directory = os.getenv(var)
    if directory:
        return Path(directory)
    else:
        return Path.home() / default


def get_cache_dir():
    return _resolve("XDG_CACHE_HOME", ".cache") / "tuxrun"



from typing import Optional

from tuxrun import templates
from tuxrun.devices import Device
from tuxrun.exceptions import InvalidArgument
from tuxrun.utils import compression, notnone, slugify


class AvhDevice(Device):
    flag_use_pre_run_cmd = True
    flag_cache_rootfs = True

    api_endpoint: str = "https://app.avh.arm.com/api"
    api_token: str = ""
    project_name: str = "Default Project"
    model: str = ""
    boot_args: Optional[str] = None

    arch: str = ""
    dtb: Optional[str] = None
    kernel: Optional[str] = None
    rootfs: Optional[str] = None
    rootfs_partition: int = 1

    test_character_delay: int = 0

    def validate(
        self,
        secrets,
        dtb,
        kernel,
        rootfs,
        boot_args,
        modules,
        overlays,
        parameters,
        partition,
        prompt,
        tests,
        commands,
        **kwargs,
    ):
        invalid_args = ["--" + k.replace("_", "-") for (k, v) in kwargs.items() if v]
        if len(invalid_args) > 0:
            raise InvalidArgument(
                f"Invalid option(s) for avh devices: {', '.join(sorted(invalid_args))}"
            )

        if not secrets:
            raise InvalidArgument("argument '--secrets' is required by AVH device")
        if secrets.get("avh_api_token") is None:
            raise InvalidArgument(
                "argument '--secrets' key 'avh_api_token' value is required by AVH device"
            )

        if boot_args and '"' in boot_args:
            raise InvalidArgument('argument --boot-args should not contains "')

        if prompt and '"' in prompt:
            raise InvalidArgument('argument --prompt should not contains "')

        if modules and compression(modules[0]) not in [("tar", "gz"), ("tar", "xz")]:
            raise InvalidArgument(
                "argument --modules should be a .tar.gz, tar.xz or .tgz"
            )

        for test in tests:
            test.validate(device=self, parameters=parameters, **kwargs)

    def default(self, options) -> None:
        options.dtb = notnone(options.dtb, self.dtb)
        options.kernel = notnone(options.kernel, self.kernel)
        options.rootfs = notnone(options.rootfs, self.rootfs)

    def definition(self, **kwargs):
        kwargs = kwargs.copy()

        # Options that can *not* be updated
        kwargs["model"] = self.model
        kwargs["arch"] = self.arch

        # Options that can be updated
        kwargs["api_endpoint"] = self.api_endpoint
        kwargs["model"] = self.model
        kwargs["project_name"] = self.project_name
        if kwargs["rootfs_partition"] is None:
            kwargs["rootfs_partition"] = self.rootfs_partition
        if kwargs["boot_args"] is None:
            kwargs["boot_args"] = self.boot_args
        if kwargs["tux_prompt"]:
            kwargs["tux_prompt"] = [kwargs["tux_prompt"]]
        else:
            kwargs["tux_prompt"] = []

        kwargs["command_name"] = slugify(
            kwargs.get("parameters").get("command-name", "command")
        )

        # render the template
        tests = [
            t.render(
                arch=kwargs["arch"],
                commands=kwargs["commands"],
                command_name=kwargs["command_name"],
                device=kwargs["device"],
                overlays=kwargs["overlays"],
                parameters=kwargs["parameters"],
                test_definitions=kwargs["test_definitions"],
            )
            for t in kwargs["tests"]
        ]
        return templates.jobs().get_template("avh.yaml.jinja2").render(
            **kwargs
        ) + "".join(tests)

    def device_dict(self, context):
        if self.test_character_delay:
            context["test_character_delay"] = self.test_character_delay
        return templates.devices().get_template("avh.yaml.jinja2").render(**context)


class AvhImx93(AvhDevice):
    name = "avh-imx93"

    model = "imx93"
    arch = "arm64"


class AvhRpi4b(AvhDevice):
    name = "avh-rpi4b"

    model = "rpi4b"
    arch = "arm64"



# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

from tuxrun.tests import Test


class LTPTest(Test):
    devices = ["qemu-*", "fvp-aemva", "avh-imx93", "avh-rpi4b"]
    cmdfile: str = ""
    need_test_definition = True

    def render(self, **kwargs):
        kwargs["name"] = self.name
        kwargs["timeout"] = self.timeout
        kwargs["cmdfile"] = (
            self.cmdfile if self.cmdfile else self.name.replace("ltp-", "")
        )

        return self._render("ltp.yaml.jinja2", **kwargs)


class LTPCapBounds(LTPTest):
    name = "ltp-cap_bounds"
    timeout = 2


class LTPCommands(LTPTest):
    name = "ltp-commands"
    timeout = 10


class LTPContainers(LTPTest):
    name = "ltp-containers"
    timeout = 5


class LTPController(LTPTest):
    name = "ltp-controllers"
    timeout = 90


class LTPCrypto(LTPTest):
    name = "ltp-crypto"
    timeout = 10


class LTPCVE(LTPTest):
    name = "ltp-cve"
    timeout = 60


class LTPDIO(LTPTest):
    name = "ltp-dio"
    timeout = 2


class LTPFcntlLockTests(LTPTest):
    name = "ltp-fcntl-locktests"
    timeout = 2


class LTPFileCaps(LTPTest):
    name = "ltp-filecaps"
    timeout = 2


class LTPFSBind(LTPTest):
    name = "ltp-fs_bind"
    timeout = 25


class LTPFSPermsSimple(LTPTest):
    name = "ltp-fs_perms_simple"
    timeout = 2


class LTPFSX(LTPTest):
    name = "ltp-fsx"
    timeout = 1


class LTPFS(LTPTest):
    name = "ltp-fs"
    timeout = 45


class LTPHugetlb(LTPTest):
    name = "ltp-hugetlb"
    timeout = 2


class LTPIO(LTPTest):
    name = "ltp-io"
    timeout = 2


class LTPIPC(LTPTest):
    name = "ltp-ipc"
    timeout = 2


class LTPMath(LTPTest):
    name = "ltp-math"
    timeout = 2


class LTPMM(LTPTest):
    name = "ltp-mm"
    timeout = 20


class LTPNPTL(LTPTest):
    name = "ltp-nptl"
    timeout = 15


class LTPPTY(LTPTest):
    name = "ltp-pty"
    timeout = 10


class LTPSched(LTPTest):
    name = "ltp-sched"
    timeout = 10


class LTPSecurebits(LTPTest):
    name = "ltp-securebits"
    timeout = 2


class LTPSmoke(LTPTest):
    name = "ltp-smoke"
    cmdfile = "smoketest"
    timeout = 2


class LTPSyscalls(LTPTest):
    name = "ltp-syscalls"
    timeout = 60


class LTPTracing(LTPTest):
    name = "ltp-tracing"
    timeout = 2
